﻿XIncludeFile "OWire_Module.pbi"
XIncludeFile "WebServer.pbi"

UseModule OWire

EnableExplicit

Enumeration Window
  #WinMain
EndEnumeration

Enumeration Menu
  #Menu_Main
  #Menu_TablePopup
EndEnumeration

Enumeration Gadget
  #ComboPort
  #SpinThermoInterval
  #CheckThermoControl
  #ContainerThermoControl
  #SpinThermoMin
  #SpinThermoMax
  #Table
EndEnumeration

Enumeration
  #MItem_Log
  #MItem_Exit
  #MItem_Mode_CPU_MinUsage
  #MItem_Mode_Invert
  ;#MItem_ConvertOnly
  ;#MItem_Shem
  #MItem_About
  
  #MItem_Rename
  #MItem_DelSensor
EndEnumeration

Enumeration StatusBar
  #StatusBar_Main
EndEnumeration

Enumeration
  #StatusBar_Main_Status
  #StatusBar_Main_Server
EndEnumeration

Enumeration
  #Timer_Server
EndEnumeration

Enumeration
  #Table_Name
  #Table_Type
  #Table_SN
  #Table_Power
  #Table_Thermo
  #Table_Status
EndEnumeration

Enumeration Image
  #ImageShem
EndEnumeration

Enumeration File
  #LogFileID
EndEnumeration

#Table_Color_Err  = $8888FD
#Table_Color_Warn = $ABF9FF

#DI_ErrFlag_CRC_ROM  = 1
#DI_ErrFlag_CRC_RAM  = 1<<1
#DI_ErrFlag_Thermo   = 1<<2
#DI_ErrFlag_OW       = 1<<3
#DI_ErrFlag_NotFound = 1<<4 ; Датчик не найден.


#Event_GUI_Update = #PB_Event_FirstCustomValue + 2

#GUI_Update_Table_Flag_Text  = 1
#GUI_Update_Table_Flag_Color = 1<<1

#GUI_Update_Flag_Table = 1
#GUI_Update_Flag_StatusBar = 1<<1

CompilerIf #PB_Compiler_OS = #PB_OS_Windows
  #PortName = "COM"
CompilerElse
  #PortName = "/dev/ttyS"
CompilerEndIf

Structure DevicesInfo
  SN.OW_SN
  Thermo.f
  ErrFlag.a
  ;Name.s
EndStructure


Structure ThreadParam
  BreakTh.a      ; Если #True, то нужно завершить работу потока.
  PortChange.a   ; Если #True, то выбран другой порт.
  ThermoControl.a
  ConvertOnly.a
  ThermoInterval.u ; Интервал в секундах опроса датчиков.
  ThermoMin.w
  ThermoMax.w
  Log.a
  Port.s
  List SensorList.OW_SN()
EndStructure

Structure GUI_Update_Table_Item
  Text.s
  Color.l
  Flags.a
EndStructure

Structure GUI_Update_Table
  Item.GUI_Update_Table_Item[#Table_Status+2]
  SN.OW_SN
  IsFound.a
EndStructure

Structure GUI_Update
  Mutex.i
  List Table.GUI_Update_Table()
  StatusBarText.s
  Flags.a
EndStructure


Define Thread_OW_ID, Event, Window, x
Define Drag_TablePos=-1, String.s


Global NewList g_TableList.GUI_Update_Table()
Global gOW_Thread.ThreadParam
Global gGUI_Update.GUI_Update\Mutex = CreateMutex()
gOW_Thread\BreakTh    = #False
gOW_Thread\PortChange = #False;#True

Procedure.s GetLastErrorString(Err)
  Protected String.s
  
  Select Err
    Case #OW_Err_OK       : String = "Нет ошибок"
    Case #OW_Err_Port     : String = "Нет открытого COM порта"
    Case #OW_Err_NoDevice : String = "Нет ответа от '1-Wire' устройств"
    Case #OW_Err_IO       : String = "Ошибка обмена данными"
    Case #OW_Err_BaudRate : String = "Ошибка изменения скорости порта"
    Case #OW_Err_Search   : String = "Ошибка при поиске '1-Wire' устройств"
    Case #OW_Err_BadParam : String = "Ошибка в аргументе функции"
    Case #OW_Err_NoData   : String = "Нет данных"
  EndSelect
  
  ProcedureReturn String
EndProcedure

Procedure WriteLog(String.s)
  If IsFile(#LogFileID)=0
    If OpenFile(#LogFileID, "DS18B20_ThermoMon.log")
      If Lof(#LogFileID)<=0
        WriteStringFormat(#LogFileID, #PB_UTF8)
      Else
        FileSeek(#LogFileID, Lof(#LogFileID))
      EndIf
      Goto _next
    EndIf
  Else
    _next:
    If String<>""
      ReplaceString(String, Chr($a), " ", #PB_String_InPlace)
      ReplaceString(String, Chr($d), " ", #PB_String_InPlace)
      String = FormatDate("[%dd/%mm/%yyyy %hh:%ii:%ss]  ", Date())+ String
    EndIf
    WriteStringN(#LogFileID, String, #PB_UTF8)
    FlushFileBuffers(#LogFileID)
  EndIf
  
  
EndProcedure

Procedure.s wordform(num.i, form_one.s, form_two.s, form_three.s) 
  num % 100 
  Protected num_two.i = num % 10 
  If num > 10 And num < 20 
    ProcedureReturn form_three 
  EndIf 
  If num_two > 1 And num_two < 5 
    ProcedureReturn form_two 
  EndIf 
  If num_two = 1
    ProcedureReturn form_one 
  EndIf 
  ProcedureReturn form_three 
EndProcedure

Procedure.s SN_Text(*x.OW_SN)
  Protected String.s, i
  
  String = RSet(Hex(*x\Type), 2, "0")+"-"
  
  For i=5 To 0 Step -1
    String + RSet(Hex(*x\Number[i]), 2, "0")
  Next i
  
  String + "-"+RSet(Hex(*x\CRC), 2, "0")
  
  ProcedureReturn String
EndProcedure


Macro SetTableText(Object, Pos, String)
  Object\Table()\Item[Pos]\Text = String
  Object\Table()\Item[Pos]\Flags | #GUI_Update_Table_Flag_Text
  Object\Flags | #GUI_Update_Flag_Table
EndMacro

Macro SetTableColor(Object, Pos, Color_)
  Object\Table()\Item[Pos]\Color = Color_
  Object\Table()\Item[Pos]\Flags | #GUI_Update_Table_Flag_Color
  Object\Flags | #GUI_Update_Flag_Table
EndMacro

Macro SetStatusBarText(Object, String)
  Object\StatusBarText = String
  Object\Flags | #GUI_Update_Flag_StatusBar
EndMacro


; Procedure UpdateTableAll(List Devices.DevicesInfo())
;   Protected String.s, i=0, Err 
;   
;   ClearGadgetItems(#Table)
;   ForEach Devices()
;     String=Chr(10)
;     Select Devices()\SN\Type
;       Case $10
;         String+"DS1820 / DS18S20"
;       Case $28
;         String+"DS18B20"
;       Default
;         String+"Неизвестное устройство"
;     EndSelect
;     
;     Err=#False
;     ;Devices()\SN\crc+2
;     String + Chr(10)+SN_Text(@Devices()\SN)
;     If OW_GetCRC(Devices()\SN, 7) <> Devices()\SN\CRC
;       String+"  ОШИБКА CRC" : Err=#True
;       Devices()\ErrFlag | #DI_ErrFlag_CRC_ROM
;     EndIf
;     
;     String+Chr(10)
;     
;     If OW_SelectROM(Devices()\SN)
;       If OW_ReadPowerSupply()
;         String+"3 вывода"
;       Else
;         String+"2 вывода ('паразитное' питание)"
;       EndIf
;     Else
;       String+"Ошибка"
;     EndIf
;     
;     If Devices()\SN\Type=$10 Or Devices()\SN\Type=$28
;       String+Chr(10)+"Измерение"+Chr(10)+"Инициализация"
;     EndIf
;     AddGadgetItem(#Table, i, String)
;     If Err=#True
;       SetGadgetItemColor(#Table, i, #PB_Gadget_BackColor, #Table_Color_Err, #Table_SN)
;     EndIf
;     i+1
;   Next
;   
; EndProcedure

Procedure UpdateTableAll(List Devices.DevicesInfo(), *GUI.GUI_Update)
  Protected String.s, i
  
  ClearList(*GUI\Table())
  ForEach Devices()
    
    i+1
    
    If AddElement(*GUI\Table())
      
      *GUI\Table()\SN = Devices()\SN
      
      Select Devices()\SN\Type
        Case $10
          String="DS1820 / DS18S20"
        Case $28
          String="DS18B20"
        Default
          String="Неизвестное устройство"
      EndSelect
      
      
      ;SetTableText(*GUI, #Table_Type, "("+i+")  "+String)
      SetTableText(*GUI, #Table_Type, String)
      
      String=SN_Text(@Devices()\SN)
      If OW_GetCRC(Devices()\SN, 7) <> Devices()\SN\CRC
        String+"  ОШИБКА CRC"
        Devices()\ErrFlag | #DI_ErrFlag_CRC_ROM
        SetTableColor(*GUI, #Table_SN, #Table_Color_Err)
      EndIf
      
      SetTableText(*GUI, #Table_SN, String)
      
      If OW_SelectROM(Devices()\SN)
        If OW_ReadPowerSupply()
          String="3 вывода"
        Else
          String="2 вывода ('паразитное' питание)"
        EndIf
      Else
        String="Ошибка"
      EndIf
      
      SetTableText(*GUI, #Table_Power, String)
      
      If Devices()\SN\Type=$10 Or Devices()\SN\Type=$28
        SetTableText(*GUI, #Table_Thermo, "Измерение")
        SetTableText(*GUI, #Table_Status, "Инициализация")
      EndIf
      
    EndIf
  Next
  
EndProcedure

Procedure SearchSensor(List Devices.DevicesInfo(), Port.s, *GUI.GUI_Update)
  Protected Sensor.OW_SN, x=0
  
  StatusBarText(#StatusBar_Main, #StatusBar_Main_Status, "Поиск '1-Wire' устройств...")
    
  If ListSize(Devices())>0
    ClearList(Devices())
  EndIf
    
  OW_SearchROM_Reset()
  
  
  While OW_SearchROM_Enum(@Sensor)=#True
    x+1
    If AddElement(Devices())
      CopyStructure(@Sensor, @Devices()\SN, OW_SN)
    EndIf
  Wend
    
  If x>0 
    UpdateTableAll(Devices(), *GUI)
    StatusBarText(#StatusBar_Main, #StatusBar_Main_Status,
                  "Найдено "+x+" '1-Wire' "+wordform(x, "устройство", " устройства", " устройств")+".")
  Else
    ;ClearGadgetItems(#Table)
    StatusBarText(#StatusBar_Main, #StatusBar_Main_Status,
                  "На порту "+Port+", '1-Wire' устройства не найдены.")
    ClearList(*GUI\Table())
    *GUI\Flags | (#GUI_Update_Flag_StatusBar | #GUI_Update_Flag_Table)
  EndIf
EndProcedure


Procedure SelectPort(Port.s, List Devices.DevicesInfo(), *GUI.GUI_Update, NoSearch=#False)
   
  OW_ClosePort()
  
  If Port<>""
    If OW_OpenPort(Port)
      SetStatusBarText(*GUI, "Порт открыт")
    Else
      SetStatusBarText(*GUI, "Не удалось открыть порт '"+Port+
                          "'  Возможно он зянят или отстуствует в системе")
    EndIf
  EndIf
  
  *GUI\Flags | (#GUI_Update_Flag_StatusBar | #GUI_Update_Flag_Table)
  
  If NoSearch=#False
    ClearList(Devices())
    ClearList(*GUI\Table())
    
    If OW_GetLastError() = #OW_Err_OK   ; Ошибок нет.
      SearchSensor(Devices(), Port, *GUI)
    EndIf
    
  EndIf
  
EndProcedure


Procedure GUI_UpdateInfo(*GUI.GUI_Update)
  If *GUI\Flags
    
    LockMutex(gGUI_Update\Mutex)
    gGUI_Update\Flags=*GUI\Flags
    
    If *GUI\Flags & #GUI_Update_Flag_Table
      CopyList(*GUI\Table(), gGUI_Update\Table())
    EndIf
    
    If *GUI\Flags & #GUI_Update_Flag_StatusBar
      gGUI_Update\StatusBarText=*GUI\StatusBarText
    EndIf
    
    UnlockMutex(gGUI_Update\Mutex)
    
    PostEvent(#Event_GUI_Update, #WinMain, 0)
    
    *GUI\Flags=0
  EndIf
EndProcedure



Procedure Thread_OW(*Bool)
  Protected NewList Devices.DevicesInfo()
  Protected String.s, Count, i, Err
  Protected x.w, Thermo.f, RAM.OW_RAM
  Protected CountErr, OldCountErr
  Protected WarnCount, OldWarnCount
  Protected GUI.GUI_Update, Port.s
  
  ForEach gOW_Thread\SensorList()
    If AddElement(Devices())
      Devices()\SN = gOW_Thread\SensorList()
    EndIf
  Next
  
  Count = ListSize(Devices())
  Port=gOW_Thread\Port
  SelectPort(Port, Devices(), @GUI, #True)
  UpdateTableAll(Devices(), @GUI)
  GUI_UpdateInfo(@GUI)
  
  Repeat
    
    Delay(100)
    
    GUI\Flags = 0
    
    If gOW_Thread\PortChange
      Port=gOW_Thread\Port
      SelectPort(Port, Devices(), @GUI)
      Count = ListSize(Devices())
      gOW_Thread\PortChange = #False
      GUI_UpdateInfo(@GUI)
    EndIf
    
    If Count>0
      
      
      If OW_SkipROM()=#False
        String = GetLastErrorString(OW_GetLastError())
        SetStatusBarText(GUI, String)
        Err=#True
        Goto M1
      Else
          
        If OW_ConvertT()=#False
          String = GetLastErrorString(OW_GetLastError())
          SetStatusBarText(GUI, String)
          Err=#True
          Goto M1
        Else
          
          CountErr=0
          WarnCount=0
          
          For i=1 To 8
            Delay(100)
            If gOW_Thread\PortChange
              Break
            ElseIf gOW_Thread\BreakTh
              Break 2
            EndIf
          Next i
          
                      
            i=0
            ForEach Devices()
              
              If SelectElement(GUI\Table(), i)
                
                For x=#Table_Type To #Table_Status
                  GUI\Table()\Item[x]\Flags=0
                Next x
                
                If Devices()\SN\Type = $10 Or ; DS1820 / DS18S20
                   Devices()\SN\Type = $28    ; DS18B20
                  
                  If OW_SelectROM(@Devices()\SN)
                    If OW_ReadRAM(@RAM)
                      x=(RAM\H_Termo<<8)|RAM\L_Termo
                      If Devices()\SN\Type = $28 ; DS18B20
                        Thermo=x/16
                      Else
                        Thermo=x/2
                      EndIf
                      Devices()\Thermo = Thermo
                      SetTableText(GUI, #Table_Thermo, StrF(Thermo, 1)+" °C")
                      If gOW_Thread\Log
                        WriteLog(SN_Text(@Devices()\SN)+"  —  "+StrF(Thermo, 1)+" °C")
                      EndIf
                      
                      
                      If Thermo<-55 Or Thermo>127
                        Devices()\ErrFlag | #DI_ErrFlag_Thermo
                        SetTableColor(GUI, #Table_Thermo, #Table_Color_Warn)
                      Else
                        Devices()\ErrFlag & ~#DI_ErrFlag_Thermo
                        x/16
                        If gOW_Thread\ThermoControl=0 Or
                           (gOW_Thread\ThermoMin<=x And x<=gOW_Thread\ThermoMax)
                          SetTableColor(GUI, #Table_Thermo, -1)
                        Else
                          WarnCount+1
                          SetTableColor(GUI, #Table_Thermo, #Table_Color_Warn)
                        EndIf
                      EndIf
                      If OW_GetCRC(@RAM, 8)=RAM\CRC
                        Devices()\ErrFlag & ~#DI_ErrFlag_CRC_RAM
                        SetTableText(GUI, #Table_Status, "OK")
                        SetTableColor(GUI, #Table_Status, -1)
                      Else
                        Devices()\ErrFlag | #DI_ErrFlag_CRC_RAM
                        SetTableText(GUI, #Table_Status, "Ошибка CRC")
                        SetTableColor(GUI, #Table_Status, #Table_Color_Err)
                      EndIf
                    EndIf
                  EndIf
                  
                  If OW_GetLastError()<>#OW_Err_OK 
                    Devices()\ErrFlag | #DI_ErrFlag_OW
                    SetTableText(GUI, #Table_Status, GetLastErrorString(OW_GetLastError()))
                    SetTableColor(GUI, #Table_Status, #Table_Color_Err)
                  ElseIf Devices()\ErrFlag & #DI_ErrFlag_CRC_RAM = 0
                    Devices()\ErrFlag & ~#DI_ErrFlag_OW
                    SetTableColor(GUI, #Table_Status, -1)
                  EndIf
                  
                EndIf
                
                If Devices()\ErrFlag
                  CountErr+1
                  If Devices()\ErrFlag & ~#DI_ErrFlag_CRC_ROM
                    SetTableText(GUI, #Table_Thermo, "")
                  EndIf
                EndIf
                
              EndIf
              
              i+1
              
            Next
            
            
            
            If Err=#True Or CountErr<>OldCountErr Or OldWarnCount<>WarnCount
              
              If CountErr>0
                String=".    "+wordform(CountErr, "Обнаружен ", "Обнаружено ", "Обнаружены ")+
                       CountErr+wordform(CountErr, " датчик", " датчика", " датчиков")+
                       " с ошибками в работе"
              Else
                String="."
              EndIf
              
              If WarnCount>0
                String+"  В "+WarnCount+wordform(WarnCount, " датчике", " датчиках", " датчиках")+
                       " зафиксирован выход за пределы допустимой температуры"
              EndIf
              
              SetStatusBarText(GUI, "Найдено "+Count+" '1-Wire' "+
                            wordform(Count, "устройство", " устройства", " устройств")+String)
                            
              Err=#False
              OldCountErr = CountErr
              OldWarnCount=WarnCount
            EndIf
            
          GUI_UpdateInfo(@GUI)
          
          If gOW_Thread\ThermoInterval>1
            For i=1 To gOW_Thread\ThermoInterval-1
              For x=1 To 10
                Delay(100)
                If gOW_Thread\PortChange Or gOW_Thread\BreakTh
                  Break
                EndIf
              Next x
            Next i
          EndIf
          
          
        EndIf
      EndIf
      
      
      
    Else
      M1:
      If WarnCount<>-1
        x=ListSize(GUI\Table())-1
        For i=0 To x
          SetTableText(GUI, #Table_Thermo, " ")
          SetTableText(GUI, #Table_Status, String)
          SetTableColor(GUI, #Table_Thermo, -1)
          SetTableColor(GUI, #Table_Status, #Table_Color_Err)
        Next
        WarnCount=-1
      EndIf
      
      CountErr+1
      If CountErr>=10
        Count=0
        ;If OW_Reset()
          SearchSensor(Devices(), Port, @GUI)
          Count = ListSize(Devices())
        ;EndIf
        GUI\Flags | (#GUI_Update_Flag_StatusBar | #GUI_Update_Flag_Table)
        CountErr=0
      EndIf
      GUI_UpdateInfo(@GUI)
    EndIf
    
    
    
  Until gOW_Thread\BreakTh
    
EndProcedure



Procedure EnumPorts(Gadget)
  Protected i, ID
  
  ClearGadgetItems(Gadget)
  
  CompilerIf #PB_Compiler_OS = #PB_OS_MacOS
    AddGadgetItem(Gadget, -1, "/dev/tty.usbserial")
  CompilerElse
    
    Protected s.s
    id = ExamineDirectory(#PB_Any, "/dev/", "*.*")

    If id
      While NextDirectoryEntry(id)
        If DirectoryEntryType(id) = #PB_DirectoryEntry_File
          s = DirectoryEntryName(id)
          If FindString(s, "tty", 1, #PB_String_NoCase) = 1
            s = "/dev/"+s
            If OpenSerialPort(0, s, 9600, #PB_SerialPort_NoParity, 8, 1, 
                              #PB_SerialPort_NoHandshake, 1024, 1024)
              AddGadgetItem(Gadget, -1, s)
              ;Debug  s
              CloseSerialPort(0)
            EndIf
          EndIf
        EndIf
      Wend
      FinishDirectory(id)
    EndIf
    
;     For i=1 To 30
; ;       ID = OpenSerialPort(#PB_Any, #PortName+i, 115200, #PB_SerialPort_NoParity,
; ;                           8, 1, #PB_SerialPort_NoHandshake, 255, 255)
; ;       If ID
;         AddGadgetItem(Gadget, -1, #PortName+i)
; ;         CloseSerialPort(ID)
; ;       EndIf
;     Next
    
  CompilerEndIf

EndProcedure

Procedure OpenWinMain()
  Protected x
  
  CompilerIf #PB_Compiler_OS = #PB_OS_Linux
    SetGadgetFont(#PB_Default, FontID(LoadFont(#PB_Any,"Liberation Mono",10)))
  CompilerEndIf
  
  OpenWindow(#WinMain, 0, 0, 750, 400, "DS18B20 термо-мониторинг v1.3", 
             #PB_Window_MinimizeGadget|#PB_Window_Invisible|#PB_Window_ScreenCentered)
  
  CreateMenu(#Menu_Main, WindowID(#WinMain))
  MenuTitle("Файл")
    MenuItem(#MItem_Log, "Записывать в лог-файл")
    MenuBar()
    MenuItem(#MItem_Exit, "Выход")
  MenuTitle("Режим")
    MenuItem(#MItem_Mode_CPU_MinUsage, "Снизить нагрузку на CPU")
    MenuItem(#MItem_Mode_Invert, "Инвертировать DTR и RTS")
    ;MenuItem(#MItem_ConvertOnly, "Раздельный опрос")
  MenuTitle("Справка")
    MenuItem(#MItem_About, "О программе")
  
  
  If CreatePopupMenu(#Menu_TablePopup)
    MenuItem(#MItem_Rename, "Переименовать")
    MenuItem(#MItem_DelSensor, "Удалить не найденные")
  EndIf
  
  x=TextGadget(#PB_Any, 4, 8, 100, 16, "Порт:")
  ResizeGadget(x, #PB_Ignore, #PB_Ignore, GadgetWidth(x, #PB_Gadget_RequiredSize)+4, #PB_Ignore)
  
  ComboBoxGadget(#ComboPort, GadgetX(x)+GadgetWidth(x), 4, 130, 22)
  EnumPorts(#ComboPort)
  
  
  x=TextGadget(#PB_Any, GadgetX(#ComboPort)+GadgetWidth(#ComboPort)+10, 8, 350, 16, "Интервал опроса:")
  ResizeGadget(x, #PB_Ignore, #PB_Ignore, GadgetWidth(x, #PB_Gadget_RequiredSize)+4, #PB_Ignore)
  
  SpinGadget(#SpinThermoInterval, GadgetX(x)+GadgetWidth(x), 4, 70, 22, 1, 65530,
             #PB_Spin_Numeric)
  GadgetToolTip(#SpinThermoInterval, "Интервал в секундах опроса датчиков")
  
  CheckBoxGadget(#CheckThermoControl, GadgetX(#SpinThermoInterval)+GadgetWidth(#SpinThermoInterval)+16,
                 8, 100, 16, "Термо контроль")
  ResizeGadget(#CheckThermoControl, #PB_Ignore, #PB_Ignore,
               GadgetWidth(#CheckThermoControl, #PB_Gadget_RequiredSize)+2, #PB_Ignore)
  
  ContainerGadget(#ContainerThermoControl, GadgetX(#CheckThermoControl)+GadgetWidth(#CheckThermoControl),
                  2, 200, 28, #PB_Container_Flat)
  
  x=TextGadget(#PB_Any, 4, 7, 350, 16, "Мин:")
  ResizeGadget(x, #PB_Ignore, #PB_Ignore, GadgetWidth(x, #PB_Gadget_RequiredSize)+4, #PB_Ignore)
  
  SpinGadget(#SpinThermoMin, GadgetX(x)+GadgetWidth(x), 2, 50, 22, -55, 125,
             #PB_Spin_ReadOnly|#PB_Spin_Numeric)
  GadgetToolTip(#SpinThermoMin, "Минимальная допустимая температура")
  
  x=TextGadget(#PB_Any, GadgetX(#SpinThermoMin)+GadgetWidth(#SpinThermoMin)+10, 7, 350, 16, "Макс:")
  ResizeGadget(x, #PB_Ignore, #PB_Ignore, GadgetWidth(x, #PB_Gadget_RequiredSize)+4, #PB_Ignore)
  
  SpinGadget(#SpinThermoMax, GadgetX(x)+GadgetWidth(x), 2, 50, 22, -55, 125,
             #PB_Spin_ReadOnly|#PB_Spin_Numeric)
  GadgetToolTip(#SpinThermoMax, "Максимальная допустимая температура")
  
  CloseGadgetList()
  
  ResizeGadget(#ContainerThermoControl, #PB_Ignore, #PB_Ignore, 
               GadgetX(#SpinThermoMax)+GadgetWidth(#SpinThermoMax)+10, #PB_Ignore)
  
  ResizeWindow(#WinMain, #PB_Ignore, #PB_Ignore, 
               GadgetX(#ContainerThermoControl)+GadgetWidth(#ContainerThermoControl)+2, #PB_Ignore)
    
  x=ListIconGadget(#Table, 2, GadgetY(#ContainerThermoControl)+GadgetHeight(#ContainerThermoControl)+2, 
                   WindowWidth(#WinMain)-4, 304, "Имя", 120, 
                   #PB_ListIcon_GridLines|#PB_ListIcon_FullRowSelect|#PB_ListIcon_AlwaysShowSelection)
  
  EnableGadgetDrop(#Table, #PB_Drop_Private, #PB_Drag_Move, #Table)
  
  CompilerIf #PB_Compiler_OS = #PB_OS_Windows
    SendMessage_(x, #LVM_SETEXTENDEDLISTVIEWSTYLE, 
                 #LVS_EX_LABELTIP|#LVS_EX_GRIDLINES|#LVS_EX_DOUBLEBUFFER, 
                 #LVS_EX_LABELTIP|#LVS_EX_GRIDLINES|#LVS_EX_DOUBLEBUFFER)
  CompilerEndIf
  
  AddGadgetColumn(#Table, #Table_Type, "Тип", 100)
  AddGadgetColumn(#Table, #Table_SN, "Серийный номер", 170)
  AddGadgetColumn(#Table, #Table_Power, "Подключение", 100)
  AddGadgetColumn(#Table, #Table_Thermo, "Температура", 100)
  AddGadgetColumn(#Table, #Table_Status, "Статус", 120)
  
  CreateStatusBar(#StatusBar_Main, WindowID(#WinMain))
  AddStatusBarField(#PB_Ignore)
  ;AddStatusBarField(#PB_Ignore)
  
  ResizeGadget(#Table, #PB_Ignore, #PB_Ignore, #PB_Ignore,
               WindowHeight(#WinMain)-StatusBarHeight(#StatusBar_Main)-GadgetY(#Table)-MenuHeight()-2)
  
  
  HideWindow(#WinMain, #False)
EndProcedure

Procedure ReadSetting()
  Protected String.s, x,Flag, p
  Protected i, TempQ.q
  
  x=OpenPreferences("ThermoMon.ini")
  
  String=ReadPreferenceString("Port", "")
  If String
    SetGadgetText(#ComboPort, String)
  Else
    SetGadgetState(#ComboPort, -1)
  EndIf
  gOW_Thread\ThermoInterval = ReadPreferenceLong("Interval", 1)
;   If gOW_Thread\ThermoInterval<=0 Or gOW_Thread\ThermoInterval>100
;     gOW_Thread\ThermoInterval=1
;   EndIf
  SetGadgetState(#SpinThermoInterval, gOW_Thread\ThermoInterval)
  SetGadgetText(#SpinThermoInterval, Str(gOW_Thread\ThermoInterval))
  
  gOW_Thread\ThermoControl = ReadPreferenceLong("ThermoControl", 0)&1
  SetGadgetState(#CheckThermoControl, gOW_Thread\ThermoControl)
  
  ;gOW_Thread\ConvertOnly = ReadPreferenceLong("ConvertOnly", 0)&1
  ;SetMenuItemState(#Menu_Main, #MItem_ConvertOnly, gOW_Thread\ConvertOnly)
  
  gOW_Thread\ThermoMin = ReadPreferenceLong("ThermoMin", 0)
  gOW_Thread\ThermoMax = ReadPreferenceLong("ThermoMax", 10)
  
  gOW_Thread\Log = ReadPreferenceLong("Log", 0)&1
  SetMenuItemState(#Menu_Main, #MItem_Log, gOW_Thread\Log)
  
  Flag = OW_GetMode()
  p = ReadPreferenceLong("CPU_MinUsage", 0)&1
  If p
    Flag|#OW_Mode_CPU_MinUsage
  EndIf
  SetMenuItemState(#Menu_Main, #MItem_Mode_CPU_MinUsage, p)
  
  p=ReadPreferenceLong("Invert", 0)
  If p
    Flag|#OW_Mode_Invert
  EndIf
  SetMenuItemState(#Menu_Main, #MItem_Mode_Invert, p)
  
  OW_SetMode(Flag)                 
  
  If gOW_Thread\ThermoMin<=-55 Or gOW_Thread\ThermoMin>130
    gOW_Thread\ThermoMin=0
  EndIf
  
  If gOW_Thread\ThermoMax<=-55 Or gOW_Thread\ThermoMax>130
    gOW_Thread\ThermoMax=10
  EndIf
  
  If gOW_Thread\ThermoMin>=gOW_Thread\ThermoMax
    gOW_Thread\ThermoMin=gOW_Thread\ThermoMax-1
  EndIf
  
  
  PreferenceGroup("Sensors")
  p=ReadPreferenceLong("Count", 0)
  If p>100 : p=100 : EndIf
  
  ClearList(g_TableList())
  
  For i=0 To p
    TempQ = ReadPreferenceQuad("ID_"+Str(i), 0)
    If TempQ
      If AddElement(g_TableList())
        PokeQ(@g_TableList()\SN, TempQ)
        g_TableList()\Item[#Table_Name]\Text = ReadPreferenceString("Name_"+Str(i), "Новый")
        g_TableList()\Item[#Table_Name]\Flags=#GUI_Update_Table_Flag_Text
        g_TableList()\Item[#Table_SN]\Text=SN_Text(@TempQ)
        g_TableList()\Item[#Table_SN]\Flags=#GUI_Update_Table_Flag_Text
      EndIf
    EndIf
  Next i
  
  PreferenceGroup("Server")
  WS::ServerEnable = Bool(ReadPreferenceLong("Enable", 0))
  WS::ServerPort   = ReadPreferenceLong("Port", 80)
  WS::IPv6         = Bool(ReadPreferenceLong("IPv6", 0))
  
  SetGadgetState(#SpinThermoMin, gOW_Thread\ThermoMin)
  SetGadgetText(#SpinThermoMin, Str(gOW_Thread\ThermoMin))
   
  
  SetGadgetState(#SpinThermoMax, gOW_Thread\ThermoMax)
  SetGadgetText(#SpinThermoMax, Str(gOW_Thread\ThermoMax))
  
  PostEvent(#PB_Event_Gadget, #WinMain, #ComboPort, #PB_EventType_Change)
  PostEvent(#PB_Event_Gadget, #WinMain, #CheckThermoControl)
  
  If x
    ClosePreferences()
  EndIf
EndProcedure

Procedure WriteSetting()
  Protected i=0
  
  If CreatePreferences("ThermoMon.ini")
    
    WritePreferenceString("Port", GetGadgetText(#ComboPort))
    WritePreferenceLong("Interval", GetGadgetState(#SpinThermoInterval))
    WritePreferenceLong("ThermoControl", GetGadgetState(#CheckThermoControl))
    ;WritePreferenceLong("ConvertOnly", GetMenuItemState(#Menu_Main, #MItem_ConvertOnly))
    WritePreferenceLong("ThermoMin", GetGadgetState(#SpinThermoMin))
    WritePreferenceLong("ThermoMax", GetGadgetState(#SpinThermoMax))
    WritePreferenceLong("Log", gOW_Thread\Log)
    WritePreferenceLong("CPU_MinUsage", GetMenuItemState(#Menu_Main, #MItem_Mode_CPU_MinUsage))
    WritePreferenceLong("Invert", GetMenuItemState(#Menu_Main, #MItem_Mode_Invert))
    
    PreferenceGroup("Sensors")
    WritePreferenceLong("Count", ListSize(g_TableList()))
    
    i=0
    ForEach g_TableList()
      i+1
      WritePreferenceQuad("ID_"+Str(i), PeekQ(@g_TableList()\SN))
      WritePreferenceString("Name_"+Str(i), g_TableList()\Item[#Table_Name]\Text)
    Next
    
    PreferenceGroup("Server")
    WritePreferenceLong("Enable", Bool(WS::ServerEnable))
    WritePreferenceLong("Port", WS::ServerPort)
    WritePreferenceLong("IPv6", Bool(WS::IPv6))
    
    ClosePreferences()
  EndIf
EndProcedure


Procedure PortChange()
  Static Work=#False
  
  If Work=#True
    ProcedureReturn
  EndIf
  
  Work=#True
  
  gOW_Thread\Port=GetGadgetText(#ComboPort)
  gOW_Thread\PortChange = #True
  WriteSetting()
  
  Work=#False
EndProcedure

Procedure GUI_Update(Update = #False)
  Static Work=#False
  Protected GUI.GUI_Update, i
  Protected CountList, CountItem
  Protected Temp, Pos
  ;Protected NewList Table.GUI_Update_Table()
  
  If Work=#True
    ProcedureReturn
  EndIf
  
  Work=#True
  
  If Update = #False
    LockMutex(gGUI_Update\Mutex)
    If gGUI_Update\Flags
      GUI\Flags=gGUI_Update\Flags
      
      If GUI\Flags & #GUI_Update_Flag_Table
        CopyList(gGUI_Update\Table(), GUI\Table())
      EndIf
      
      If GUI\Flags & #GUI_Update_Flag_StatusBar
        GUI\StatusBarText=gGUI_Update\StatusBarText
      EndIf
      
      gGUI_Update\Flags=0
    Else
      GUI\Flags=0
    EndIf
    UnlockMutex(gGUI_Update\Mutex)
  EndIf
  
  If GUI\Flags & #GUI_Update_Flag_Table Or Update = #True
    
    If Update = #False
      ForEach g_TableList()
        i=#False
        With g_TableList()
          ;If AddElement(Table())
          
          
          ForEach GUI\Table()
            If PeekQ(\SN) = PeekQ(GUI\Table()\SN)
              i=#True
              Break
            EndIf
          Next
          
          If i=#True
            ;CopyStructure(@GUI\Table(), @Table(), GUI_Update_Table)
            For i=#Table_Type To #Table_Status
              g_TableList()\Item[i]\Text = GUI\Table()\Item[i]\Text
              g_TableList()\Item[i]\Flags = GUI\Table()\Item[i]\Flags | #GUI_Update_Table_Flag_Text
              g_TableList()\Item[i]\Color = GUI\Table()\Item[i]\Color
            Next i
            \IsFound = #True
            DeleteElement(GUI\Table())
          Else
            ;\Item[#Table_SN]\Text = SN_Text(g_SensorList()\SN)
            ;\Item[#Table_SN]\Flags = #GUI_Update_Table_Flag_Text
            \IsFound = #False
            \Item[#Table_Status]\Text = "Не найден"
            \Item[#Table_Status]\Flags = #GUI_Update_Table_Flag_Text | #GUI_Update_Table_Flag_Color
            \Item[#Table_Status]\Color = #Table_Color_Err
            \Item[#Table_Type]\Flags = #GUI_Update_Table_Flag_Text
            \Item[#Table_Thermo]\Text = ""
            \Item[#Table_Thermo]\Flags = #GUI_Update_Table_Flag_Text
            \Item[#Table_Power]\Flags = #GUI_Update_Table_Flag_Text
          EndIf
          
          ;\Item[#Table_Name]\Text = g_SensorList()\Name
          \Item[#Table_Name]\Flags = #GUI_Update_Table_Flag_Text
          ;EndIf
        EndWith
      Next
      
      If ListSize(GUI\Table())>0
        LastElement(g_TableList())
        ForEach GUI\Table()
          If AddElement(g_TableList())
            g_TableList()\SN = GUI\Table()\SN
            g_TableList()\Item[#Table_Name]\Text = "Новый"
            g_TableList()\Item[#Table_Name]\Flags = #GUI_Update_Table_Flag_Text
            For i=#Table_Type To #Table_Status
              g_TableList()\Item[i]\Text = GUI\Table()\Item[i]\Text
              g_TableList()\Item[i]\Flags = GUI\Table()\Item[i]\Flags | #GUI_Update_Table_Flag_Text
              g_TableList()\Item[i]\Color = GUI\Table()\Item[i]\Color
            Next i
          EndIf
        Next
      EndIf
      
    EndIf
    
    CountList = ListSize(g_TableList())
    CountItem = CountGadgetItems(#Table)
    
    If CountList<>CountItem
      ClearGadgetItems(#Table)
      Temp = CountList-1
      For i=0 To Temp
        AddGadgetItem(#Table, i, "")
      Next i
    EndIf
    
    
    Pos=0
    ForEach g_TableList()
      With g_TableList()
        For i=#Table_Name To #Table_Status
          ;If \Item[i]\Flags & #GUI_Update_Table_Flag_Text
            If GetGadgetItemText(#Table, Pos, i) <> \Item[i]\Text
              SetGadgetItemText(#Table, Pos, \Item[i]\Text, i)
            EndIf
          ;EndIf
          If \Item[i]\Flags & #GUI_Update_Table_Flag_Color
            SetGadgetItemColor(#Table, Pos, #PB_Gadget_BackColor, \Item[i]\Color, i)
          EndIf
        Next i
        Pos+1
      EndWith
    Next
    
    
    If WS::ServerEnable
      NewList x.ws::SensorInfo()
      
      ForEach g_TableList()
        If AddElement(x())
          x()\Name = g_TableList()\Item[#Table_Name]\Text
          If LCase(g_TableList()\Item[#Table_Status]\Text) = "ok"
            x()\Temp = g_TableList()\Item[#Table_Thermo]\Text
          Else
            x()\Temp = g_TableList()\Item[#Table_Status]\Text
          EndIf
        EndIf
      Next
      
      WS::UpdateInfo(x())
      FreeList(x())
    EndIf
  
  EndIf
  
  If GUI\Flags & #GUI_Update_Flag_StatusBar
    StatusBarText(#StatusBar_Main, #StatusBar_Main_Status, GUI\StatusBarText)
  EndIf
  
  Work=#False
EndProcedure

Procedure GUI_UpdateTable()
  GUI_Update(#False)
EndProcedure


Procedure Table_DragDrop(OldPos, NewPos)
  Protected MoveType, *Point
  
  If OldPos>NewPos
    MoveType=#PB_List_Before ; Перед текущим элементом.
  Else
    MoveType=#PB_List_After  ; После текущего элемента.
  EndIf
  
  If SelectElement(g_TableList(), NewPos)
    *Point = @g_TableList()
    If *Point
      If SelectElement(g_TableList(), OldPos)
        MoveElement(g_TableList(), MoveType, *Point)
        GUI_Update(#True)
      EndIf
    EndIf
  EndIf
  
EndProcedure


SetCurrentDirectory(GetPathPart(ProgramFilename()))

OpenWinMain()

ReadSetting()

If WS::ServerEnable
  If WS::StartServer(WS::ServerPort, WS::IPv6)
    StatusBarText(#StatusBar_Main, #StatusBar_Main_Server, "Сервер работает ("+WS::ServerPort+" порт, IPv"+
                                                           Str(Bool(WS::IPv6)*2+4)+")")
    AddWindowTimer(#WinMain, #Timer_Server, 10)
    BindEvent(#PB_Event_Timer, WS::@ServerEvent(), #WinMain, #Timer_Server)
  Else
    StatusBarText(#StatusBar_Main, #StatusBar_Main_Server, "Не удалось запустить сервер ("+WS::ServerPort+" порт)")
  EndIf
EndIf

GUI_Update(#True)

Repeat : Until WindowEvent() = 0

BindGadgetEvent(#ComboPort, @PortChange(), #PB_EventType_Change)
BindEvent(#Event_GUI_Update, @GUI_UpdateTable(), #WinMain)

;CopyList(g_SensorList(), gOW_Thread\SensorList())
ForEach g_TableList()
  If AddElement(gOW_Thread\SensorList())
    gOW_Thread\SensorList() = g_TableList()\SN
  EndIf
Next

gOW_Thread\Port=GetGadgetText(#ComboPort)
Thread_OW_ID=CreateThread(@Thread_OW(), 0)


Repeat
  Event = WaitWindowEvent()
  Window = EventWindow()
  
  If Window = #WinMain
    If Event = #PB_Event_Menu
      Select EventMenu()
        Case #MItem_Log
          gOW_Thread\Log = GetMenuItemState(#Menu_Main, #MItem_Log)!1
          SetMenuItemState(#Menu_Main, #MItem_Log, gOW_Thread\Log)
          WriteSetting()
        Case #MItem_Exit : Break
        Case #MItem_Mode_CPU_MinUsage
          x=GetMenuItemState(#Menu_Main, #MItem_Mode_CPU_MinUsage)!1
          SetMenuItemState(#Menu_Main, #MItem_Mode_CPU_MinUsage, x)
          If x
            OW_SetMode(OW_GetMode()|#OW_Mode_CPU_MinUsage)
          Else
            OW_SetMode(OW_GetMode()&~#OW_Mode_CPU_MinUsage)
          EndIf
        Case #MItem_Mode_Invert
          x=GetMenuItemState(#Menu_Main, #MItem_Mode_Invert)!1
          SetMenuItemState(#Menu_Main, #MItem_Mode_Invert, x)
          If x
            OW_SetMode(OW_GetMode()|#OW_Mode_Invert)
          Else
            OW_SetMode(OW_GetMode()&~#OW_Mode_Invert)
          EndIf
;         Case #MItem_ConvertOnly
;           x = GetMenuItemState(#Menu_Main, #MItem_ConvertOnly) ! 1
;           gOW_Thread\ConvertOnly = x
;           SetMenuItemState(#Menu_Main, #MItem_ConvertOnly, x)
;           If x=0
;             SetGadgetItemColor(#table, -1, #PB_Gadget_BackColor, -1, #Table_Type)
;           EndIf
        Case #MItem_About
          MessageRequester("О программе", "Мониторинг температуры с использованием DS18x20."+Chr(10)+
                                          "E-Mail: pure-basic@yandex.ru")
          
        Case #MItem_Rename
          x=GetGadgetState(#Table)
          If x>=0 And x<=ListSize(g_TableList())
            If SelectElement(g_TableList(), x)
              String = InputRequester("Выбор имени датчика", "", g_TableList()\Item[#Table_Name]\Text)
              If String
                If SelectElement(g_TableList(), x)
                  g_TableList()\Item[#Table_Name]\Text = String
                  GUI_Update(#True)
                EndIf
                String=""
              EndIf
            EndIf
          EndIf
        Case #MItem_DelSensor
          ForEach g_TableList()
            If g_TableList()\IsFound = #False
              DeleteElement(g_TableList())
            EndIf
          Next
          GUI_Update(#True)
      EndSelect
      
    ElseIf Event = #PB_Event_Gadget
      Select EventGadget()
        Case #SpinThermoInterval
          gOW_Thread\ThermoInterval = GetGadgetState(#SpinThermoInterval)
          WriteSetting()
        Case #CheckThermoControl
          gOW_Thread\ThermoControl = GetGadgetState(#CheckThermoControl)
          WriteSetting()          
        Case #SpinThermoMin
          x=GetGadgetState(#SpinThermoMin)
          Select EventType()
            Case #PB_EventType_Change
              gOW_Thread\ThermoMin = x
              WriteSetting()
            Case #PB_EventType_Up
              If x>=GetGadgetState(#SpinThermoMax)
                x=GetGadgetState(#SpinThermoMax)-1
                SetGadgetState(#SpinThermoMin, x)
                SetGadgetText(#SpinThermoMin, Str(x))
              EndIf
          EndSelect
        Case #SpinThermoMax
          x=GetGadgetState(#SpinThermoMax)
          Select EventType()
            Case #PB_EventType_Change
              gOW_Thread\ThermoMax = x
              WriteSetting()
            Case #PB_EventType_Down
              If x=<GetGadgetState(#SpinThermoMin)
                x=GetGadgetState(#SpinThermoMin)+1
                SetGadgetState(#SpinThermoMax, x)
                SetGadgetText(#SpinThermoMax, Str(x))
              EndIf
          EndSelect
          
        Case #Table
          x = EventType()
          If x = #PB_EventType_DragStart
            
            Drag_TablePos=GetGadgetState(#Table)
            If Drag_TablePos>=0
              If DragPrivate(#Table, #PB_Drag_Move) <> #PB_Drag_Move
                Drag_TablePos=-1
              EndIf
            EndIf
            
          ElseIf x = #PB_EventType_RightClick
            x = GetGadgetState(#Table)
            If x>=0
              DisplayPopupMenu(#Menu_TablePopup, WindowID(#WinMain))
            EndIf
          EndIf
          
      EndSelect      
      
    ElseIf Event = #PB_Event_GadgetDrop
      If EventGadget() = #Table
        If EventDropType() = #PB_Drop_Private
          x = GetGadgetState(#Table)
          If x>=0 And Drag_TablePos>=0 And Drag_TablePos<>x
            Table_DragDrop(Drag_TablePos, x)
          EndIf
        EndIf
      EndIf
    EndIf
  EndIf
  
Until Event = #PB_Event_CloseWindow And Window = #WinMain

gOW_Thread\BreakTh=#True
HideWindow(#WinMain, #True)
WriteSetting()
WaitThread(Thread_OW_ID, 400)
If IsThread(Thread_OW_ID)
  KillThread(Thread_OW_ID)
EndIf

If IsFile(#LogFileID)
  CloseFile(#LogFileID)
EndIf

End
; IDE Options = PureBasic 6.00 LTS - C Backend (Linux - arm64)
; CursorPosition = 775
; FirstLine = 760
; Folding = ----
; Optimizer
; EnableThread
; EnableXP
; DPIAware
; UseIcon = thermometer.ico
; Executable = ../ThermoMon_1.3_arm64
; SubSystem = qt
; CompileSourceDirectory
; EnablePurifier
; IncludeVersionInfo
; VersionField0 = 1.2.1.0
; VersionField1 = 1.2.1.0
; VersionField2 = Пётр
; VersionField3 = DS18B20 ThermoMon
; VersionField4 = 1.2
; VersionField5 = 1.2
; VersionField6 = DS18B20 термо-мониторинг
; VersionField7 = DS18B20 ThermoMon
; VersionField8 = %EXECUTABLE
; VersionField13 = pure-basic@yandex.ru
; VersionField15 = VOS_NT_WINDOWS32
; VersionField16 = VFT_APP
; VersionField17 = 0419 Russian
; EnableUnicode